@@ -48,7 +48,6 @@ class User(peewee.Model):
 
 
 class Relationship(peewee.Model):
-    database = database
     from_user = peewee.ForeignKeyField(User, related_name='relationships')
     to_user = peewee.ForeignKeyField(User, related_name='related_to')
 
@@ -57,7 +56,6 @@ class Relationship(peewee.Model):
 
 
 class Message(peewee.Model):
-    database = database
     user = peewee.ForeignKeyField(User)
     content = peewee.TextField()
     pub_date = peewee.DateTimeField()
@@ -122,8 +120,9 @@ def homepage():
 @app.route('/private/')
 def private_timeline():
     user = session['user']
-    following_ids = [u.id for u in user.following()]
-    messages = Message.select().where(user_id__in=following_ids).order_by(('pub_date', 'desc'))
+    messages = Message.select().where(
+        user__in=user.following()
+    ).order_by(('pub_date', 'desc'))
     return object_list('private_messages.html', messages, 'message_list')
 
 @app.route('/public/')
@@ -204,7 +203,7 @@ def user_follow(username):
         user = User.get(username=username)
     except StopIteration:
         abort(404)
-    r = Relationship.get_or_create(
+    Relationship.get_or_create(
         from_user=session['user'],
         to_user=user,
     )
