@@ -10,6 +10,7 @@
 from datetime import datetime
 import logging
 import os
+import psycopg2
 import re
 import sqlite3
 import time
@@ -19,15 +20,128 @@ DATABASE_NAME = os.environ.get('PEEWEE_DATABASE', 'peewee.db')
 logger = logging.getLogger('peewee.logger')
 
 
+class BaseAdapter(object):
+    operations = {'eq': '= %s'}
+    interpolation = '%s'
+    
+    def get_field_types(self):
+        field_types = {
+            'integer': 'INTEGER',
+            'float': 'REAL',
+            'decimal': 'NUMERIC',
+            'string': 'VARCHAR',
+            'text': 'TEXT',
+            'datetime': 'DATETIME',
+            'primary_key': 'INTEGER',
+            'foreign_key': 'INTEGER',
+            'boolean': 'SMALLINT',
+        }
+        field_types.update(self.get_field_overrides())
+        return field_types
+    
+    def get_field_overrides(self):
+        return {}
+    
+    def connect(self, database, **kwargs):
+        raise NotImplementedError
+    
+    def close(self, conn):
+        conn.close()
+    
+    def lookup_cast(self, lookup, value):
+        return value
+    
+    def last_insert_id(self, cursor, model):
+        raise NotImplementedError
+    
+    def rows_affected(self, cursor):
+        return cursor.rowcount
+
+
+class SqliteAdapter(BaseAdapter):
+    operations = {
+        'lt': '< ?',
+        'lte': '<= ?',
+        'gt': '> ?',
+        'gte': '>= ?',
+        'eq': '= ?',
+        'ne': '!= ?', # watch yourself with this one
+        'in': 'IN (%s)', # special-case to list q-marks
+        'is': 'IS ?',
+        'icontains': "LIKE ? ESCAPE '\\'", # surround param with %'s
+        'contains': "GLOB ?", # surround param with *'s
+        'istartswith': "LIKE ? ESCAPE '\\'",
+        'startswith': "GLOB ?",
+    }
+    interpolation = '?'
+    
+    def connect(self, database, **kwargs):
+        return sqlite3.connect(database, **kwargs)
+    
+    def last_insert_id(self, cursor, model):
+        return cursor.lastrowid
+    
+    def lookup_cast(self, lookup, value):
+        if lookup == 'contains':
+            return '*%s*' % value
+        elif lookup == 'icontains':
+            return '%%%s%%' % value
+        elif lookup == 'startswith':
+            return '%s*' % value
+        elif lookup == 'istartswith':
+            return '%s%%' % value
+        return value
+
+
+class PostgresqlAdapter(BaseAdapter):
+    operations = {
+        'lt': '< %s',
+        'lte': '<= %s',
+        'gt': '> %s',
+        'gte': '>= %s',
+        'eq': '= %s',
+        'ne': '!= %s', # watch yourself with this one
+        'in': 'IN (%s)', # special-case to list q-marks
+        'is': 'IS %s',
+        'icontains': 'ILIKE %s', # surround param with %'s
+        'contains': 'LIKE %s', # surround param with *'s
+        'istartswith': 'ILIKE %s',
+        'startswith': 'LIKE %s',
+    }
+        
+    def connect(self, database, **kwargs):
+        return psycopg2.connect(database=database, **kwargs)
+    
+    def get_field_overrides(self):
+        return {
+            'primary_key': 'SERIAL',
+            'datetime': 'TIMESTAMP WITH TIME ZONE'
+        }
+    
+    def last_insert_id(self, cursor, model):
+        cursor.execute("SELECT CURRVAL('\"%s_%s_seq\"')" % (
+            model._meta.db_table, model._meta.pk_name))
+        return cursor.fetchone()[0]
+    
+    def lookup_cast(self, lookup, value):
+        if lookup in ('contains', 'icontains'):
+            return '%%%s%%' % value
+        elif lookup in ('startswith', 'istartswith'):
+            return '%s%%' % value
+        return value
+
+
 class Database(object):
-    def __init__(self, database):
+    def __init__(self, adapter, database, **connect_kwargs):
+        self.adapter = adapter
         self.database = database
+        self.connect_kwargs = connect_kwargs
     
     def connect(self):
-        self.conn = sqlite3.connect(self.database)
+        self.conn = self.adapter.connect(self.database, **self.connect_kwargs)
     
     def close(self):
-        self.conn.close()
+        self.adapter.close(self.conn)
     
     def execute(self, sql, params=None, commit=False):
         cursor = self.conn.cursor()
@@ -35,14 +149,24 @@ class Database(object):
         if commit:
             self.conn.commit()
         logger.debug((sql, params))
-        return res
+        return cursor
     
-    def last_insert_id(self):
-        result = self.execute("SELECT last_insert_rowid();")
-        return result.fetchone()[0]
+    def last_insert_id(self, cursor, model):
+        return self.adapter.last_insert_id(cursor, model)
+    
+    def rows_affected(self, cursor):
+        return self.adapter.rows_affected(cursor)
+    
+    def column_for_field(self, db_field):
+        try:
+            return self.adapter.get_field_types()[db_field]
+        except KeyError:
+            raise AttributeError('Unknown field type: "%s", valid types are: %s' % \
+                db_field, ', '.join(self.adapter.get_field_types().keys())
+            )
     
     def create_table(self, model_class):
-        framing = "CREATE TABLE IF NOT EXISTS %s (%s);"
+        framing = "CREATE TABLE %s (%s);"
         columns = []
 
         for field in model_class._meta.fields.values():
@@ -52,11 +176,29 @@ class Database(object):
         
         self.execute(query, commit=True)
     
+    def create_index(self, model_class, field, unique=False):
+        framing = 'CREATE %(unique)s INDEX %(model)s_%(field)s ON %(model)s(%(field)s);'
+        
+        if field not in model_class._meta.fields:
+            raise AttributeError(
+                'Field %s not on model %s' % (field, model_class)
+            )
+        
+        unique_expr = ternary(unique, 'UNIQUE', '')
+        
+        query = framing % {
+            'unique': unique_expr,
+            'model': model_class._meta.db_table,
+            'field': field
+        }
+        
+        self.execute(query, commit=True)
+    
     def drop_table(self, model_class):
         self.execute('DROP TABLE %s;' % model_class._meta.db_table, commit=True)
 
 
-database = Database(DATABASE_NAME)
+database = Database(SqliteAdapter(), DATABASE_NAME)
 
 
 class QueryResultWrapper(object):
@@ -120,6 +262,10 @@ def mark_query_dirty(func):
         return func(self, *args, **kwargs)
     return inner
 
+# helpers
+ternary = lambda cond, t, f: (cond and [t] or [f])[0]
+
+
 class Node(object):
     def __init__(self, connector='AND'):
         self.connector = connector
@@ -128,14 +274,16 @@ class Node(object):
     
     def connect(self, rhs, connector):
         if isinstance(rhs, Q):
-            self.connector = connector
-            self.children.append(rhs)
-            return self
+            if connector == self.connector:
+                self.children.append(rhs)
+                return self
+            else:
+                p = Node(connector)
+                p.children = [self, rhs]
+                return p
         elif isinstance(rhs, Node):
-            p = Node()
-            p.connector = connector
-            p.children.append(self)
-            p.children.append(rhs)
+            p = Node(connector)
+            p.children = [self, rhs]
             return p
     
     def __or__(self, rhs):
@@ -170,17 +318,17 @@ class Q(object):
         self.parent = None
         self.negated = False
     
-    def connect(self):
+    def connect(self, connector):
         if self.parent is None:
-            self.parent = Node()
+            self.parent = Node(connector)
             self.parent.children.append(self)
     
     def __or__(self, rhs):
-        self.connect()
+        self.connect('OR')
         return self.parent | rhs
     
     def __and__(self, rhs):
-        self.connect()
+        self.connect('AND')
         return self.parent & rhs
     
     def __invert__(self):
@@ -215,18 +363,6 @@ def parseq(*args, **kwargs):
 
 
 class BaseQuery(object):
-    operations = {
-        'lt': '< ?',
-        'lte': '<= ?',
-        'gt': '> ?',
-        'gte': '>= ?',
-        'eq': '= ?',
-        'ne': '!= ?', # watch yourself with this one
-        'in': 'IN (%s)', # special-case to list q-marks
-        'is': 'IS ?',
-        'icontains': "LIKE ? ESCAPE '\\'", # surround param with %'s
-        'contains': "GLOB ?", # surround param with *'s
-    }
     query_separator = '__'
     requires_commit = True
     force_alias = False
@@ -234,10 +370,17 @@ class BaseQuery(object):
     def __init__(self, model):
         self.model = model
         self.query_context = model
+        self.database = self.model._meta.database
+        self.operations = self.database.adapter.operations
+        self.interpolation = self.database.adapter.interpolation
+        
         self._where = {}
         self._joins = []
         self._dirty = True
     
+    def lookup_cast(self, lookup, value):
+        return self.database.adapter.lookup_cast(lookup, value)
+    
     def parse_query_args(self, model, **query):
         parsed = {}
         for lhs, rhs in query.iteritems():
@@ -253,21 +396,26 @@ class BaseQuery(object):
                 if field is None:
                     raise
                 if isinstance(rhs, Model):
-                    rhs = rhs.id
+                    rhs = rhs.get_pk()
             
             if op == 'in':
                 if isinstance(rhs, SelectQuery):
                     lookup_value = rhs
                     operation = 'IN (%s)'
                 else:
-                    lookup_value = [field.lookup_value(op, o) for o in rhs]
+                    lookup_value = [field.db_value(o) for o in rhs]
                     operation = self.operations[op] % \
-                        (','.join(['?' for v in lookup_value]))
+                        (','.join([self.interpolation for v in lookup_value]))
+            elif op == 'is':
+                if rhs is not None:
+                    raise ValueError('__is lookups only accept None')
+                operation = 'IS NULL'
+                lookup_value = []
             else:
-                lookup_value = field.lookup_value(op, rhs)
+                lookup_value = field.db_value(rhs)
                 operation = self.operations[op]
             
-            parsed[field.name] = (operation, lookup_value)
+            parsed[field.name] = (operation, self.lookup_cast(op, lookup_value))
         
         return parsed
     
@@ -332,10 +480,10 @@ class BaseQuery(object):
                 field = from_model._meta.get_related_field_for_model(model, on)
                 if field:
                     left_field = field.name
-                    right_field = 'id'                        
+                    right_field = model._meta.pk_name
                 else:
                     field = from_model._meta.get_reverse_related_field_for_model(model, on)
-                    left_field = 'id'
+                    left_field = from_model._meta.pk_name
                     right_field = field.name
                 
                 if join_type is None:
@@ -354,11 +502,12 @@ class BaseQuery(object):
                     )
                 )
         
-        for model in self._where:
-            for node in self._where[model]:
-                query, data = self.parse_node(node, model, alias_map)
-                where_with_alias.append(query)
-                where_data.extend(data)
+        for (model, join_type, on) in joins:
+            if model in self._where:
+                for node in self._where[model]:
+                    query, data = self.parse_node(node, model, alias_map)
+                    where_with_alias.append(query)
+                    where_data.extend(data)
         
         return computed_joins, where_with_alias, where_data, alias_map
     
@@ -408,7 +557,7 @@ class BaseQuery(object):
         return query, query_data
 
     def convert_subquery(self, subquery):
-        subquery.query, orig_query = 'id', subquery.query
+        subquery.query, orig_query = subquery.model._meta.pk_name, subquery.query
         subquery.force_alias, orig_alias = True, subquery.force_alias
         sql, data = subquery.sql()
         subquery.query = orig_query
@@ -417,9 +566,7 @@ class BaseQuery(object):
     
     def raw_execute(self):
         query, params = self.sql()
-        db = self.model._meta.database
-        result = db.execute(query, params, self.requires_commit)
-        return result
+        return self.database.execute(query, params, self.requires_commit)
 
 
 class SelectQuery(BaseQuery):
@@ -447,9 +594,9 @@ class SelectQuery(BaseQuery):
         tmp_query = self.query
         
         if self.use_aliases():
-            self.query = 'COUNT(t1.id)'
+            self.query = 'COUNT(t1.%s)' % (self.model._meta.pk_name)
         else:
-            self.query = 'COUNT(id)'
+            self.query = 'COUNT(%s)' % (self.model._meta.pk_name)
         
         db = self.model._meta.database
         res = db.execute(*self.sql())
@@ -489,22 +636,23 @@ class SelectQuery(BaseQuery):
 
     def parse_select_query(self, alias_map):
         if isinstance(self.query, basestring):
-            if self.query in ('*', 'id') and self.use_aliases():
+            if self.query in ('*', self.model._meta.pk_name) and self.use_aliases():
                 return '%s.%s' % (alias_map[self.model], self.query)
             return self.query
         elif isinstance(self.query, dict):
             qparts = []
+            aggregates = []
             for model, cols in self.query.iteritems():
                 alias = alias_map.get(model, '')
                 for col in cols:
                     if isinstance(col, tuple):
                         func, col, col_alias = col
-                        qparts.append('%s(%s) AS %s' % \
+                        aggregates.append('%s(%s) AS %s' % \
                             (func, self.combine_field(alias, col), col_alias)
                         )
                     else:
                         qparts.append(self.combine_field(alias, col))
-            return ', '.join(qparts)
+            return ', '.join(qparts + aggregates)
         else:
             raise TypeError('Unknown type encountered parsing select query')
     
@@ -591,7 +739,7 @@ class UpdateQuery(BaseQuery):
         sets = {}
         for k, v in self.update_query.iteritems():
             field = self.model._meta.get_field_by_name(k)
-            sets[field.name] = field.lookup_value(None, v)
+            sets[field.name] = field.db_value(v)
         
         return sets
     
@@ -604,7 +752,7 @@ class UpdateQuery(BaseQuery):
 
         for k, v in set_statement.iteritems():
             params.append(v)
-            update_params.append('%s=?' % k)
+            update_params.append('%s=%s' % (k, self.interpolation))
         
         update = 'UPDATE %s SET %s' % (
             self.model._meta.db_table, ', '.join(update_params))
@@ -623,7 +771,7 @@ class UpdateQuery(BaseQuery):
     
     def execute(self):
         result = self.raw_execute()
-        return result.rowcount
+        return self.database.rows_affected(result)
 
 
 class DeleteQuery(BaseQuery):
@@ -648,7 +796,7 @@ class DeleteQuery(BaseQuery):
     
     def execute(self):
         result = self.raw_execute()
-        return result.rowcount
+        return self.database.rows_affected(result)
 
 
 class InsertQuery(BaseQuery):
@@ -662,7 +810,7 @@ class InsertQuery(BaseQuery):
         for k, v in self.insert_query.iteritems():
             field = self.model._meta.get_field_by_name(k)
             cols.append(k)
-            vals.append(field.lookup_value(None, v))
+            vals.append(field.db_value(v))
         
         return cols, vals
     
@@ -670,7 +818,10 @@ class InsertQuery(BaseQuery):
         cols, vals = self.parse_insert()
         
         insert = 'INSERT INTO %s (%s) VALUES (%s)' % (
-            self.model._meta.db_table, ','.join(cols), ','.join('?' for v in vals))
+            self.model._meta.db_table,
+            ','.join(cols),
+            ','.join(self.interpolation for v in vals)
+        )
         
         return insert, vals
     
@@ -682,33 +833,43 @@ class InsertQuery(BaseQuery):
     
     def execute(self):
         result = self.raw_execute()
-        return result.lastrowid
+        return self.database.last_insert_id(result, self.model)
 
 
 class Field(object):
     db_field = ''
-    field_template = "%(db_field)s"
+    field_template = "%(column_type)s%(nullable)s"
 
     def get_attributes(self):
         return {}
     
-    def __init__(self, *args, **kwargs):
+    def __init__(self, null=False, db_index=False, *args, **kwargs):
+        self.null = null
+        self.db_index = db_index
         self.attributes = self.get_attributes()
-        if 'db_field' not in kwargs:
-            kwargs['db_field'] = self.db_field
+        
+        kwargs['nullable'] = ternary(self.null, '', ' NOT NULL')
         self.attributes.update(kwargs)
     
     def add_to_class(self, klass, name):
         self.name = name
+        self.model = klass
         setattr(klass, name, None)
     
     def render_field_template(self):
+        col_type = self.model._meta.database.column_for_field(self.db_field)
+        self.attributes['column_type'] = col_type
         return self.field_template % self.attributes
     
     def to_sql(self):
         rendered = self.render_field_template()
         return '"%s" %s' % (self.name, rendered)
     
+    def null_wrapper(self, value, default=None):
+        if (self.null and value is None) or default is None:
+            return value
+        return value or default
+    
     def db_value(self, value):
         return value
     
@@ -720,13 +881,15 @@ class Field(object):
 
 
 class CharField(Field):
-    db_field = 'VARCHAR'
-    field_template = "%(db_field)s(%(max_length)d) NOT NULL"
+    db_field = 'string'
+    field_template = '%(column_type)s(%(max_length)d)%(nullable)s'
     
     def get_attributes(self):
         return {'max_length': 255}
     
     def db_value(self, value):
+        if self.null and value is None:
+            return value
         value = value or ''
         return value[:self.attributes['max_length']]
     
@@ -740,10 +903,10 @@ class CharField(Field):
     
 
 class TextField(Field):
-    db_field = 'TEXT'
+    db_field = 'text'
     
     def db_value(self, value):
-        return value or ''
+        return self.null_wrapper(value, '')
     
     def lookup_value(self, lookup_type, value):
         if lookup_type == 'contains':
@@ -755,30 +918,28 @@ class TextField(Field):
 
 
 class DateTimeField(Field):
-    db_field = 'DATETIME'
-    field_template = "%(db_field)s"
+    db_field = 'datetime'
     
     def python_value(self, value):
         if value is not None:
             value = value.rsplit('.', 1)[0]
             return datetime(*time.strptime(value, '%Y-%m-%d %H:%M:%S')[:6])
-    
-    def db_value(self, value):
-        return value or None
 
 
 class IntegerField(Field):
-    db_field = 'INTEGER'
-    field_template = "%(db_field)s NOT NULL"
+    db_field = 'integer'
     
     def db_value(self, value):
-        return value or 0
+        return self.null_wrapper(value, 0)
     
     def python_value(self, value):
-        return int(value or 0)
+        if value is not None:
+            return int(value)
 
 
 class BooleanField(IntegerField):
+    db_field = 'boolean'
+    
     def db_value(self, value):
         if value:
             return 1
@@ -789,18 +950,19 @@ class BooleanField(IntegerField):
 
 
 class FloatField(Field):
-    db_field = 'REAL'
-    field_template = "%(db_field)s NOT NULL"
+    db_field = 'float'
     
     def db_value(self, value):
-        return value or 0.0
+        return self.null_wrapper(value, 0.0)
     
     def python_value(self, value):
-        return float(value or 0)
+        if value is not None:
+            return float(value)
 
 
 class PrimaryKeyField(IntegerField):
-    field_template = "%(db_field)s NOT NULL PRIMARY KEY"
+    db_field = 'primary_key'
+    field_template = "%(column_type)s NOT NULL PRIMARY KEY"
 
 
 class ForeignRelatedObject(object):    
@@ -812,13 +974,13 @@ class ForeignRelatedObject(object):
     def __get__(self, instance, instance_type=None):
         if not getattr(instance, self.cache_name, None):
             id = getattr(instance, self.field_name, 0)
-            qr = self.to.select().where(id=id).execute()
+            qr = self.to.select().where(**{self.to._meta.pk_name: id}).execute()
             setattr(instance, self.cache_name, qr.next())
         return getattr(instance, self.cache_name)
     
     def __set__(self, instance, obj):
         assert isinstance(obj, self.to), "Cannot assign %s, invalid type" % obj
-        setattr(instance, self.field_name, obj.id)
+        setattr(instance, self.field_name, obj.get_pk())
         setattr(instance, self.cache_name, obj)
 
 
@@ -828,28 +990,29 @@ class ReverseForeignRelatedObject(object):
         self.related_model = related_model
     
     def __get__(self, instance, instance_type=None):
-        query = {self.field_name: instance.id}
+        query = {self.field_name: instance.get_pk()}
         qr = self.related_model.select().where(**query)
         return qr
 
 
 class ForeignKeyField(IntegerField):
-    field_template = '%(db_field)s %(nullable)s REFERENCES "%(to_table)s" ("id")'
+    db_field = 'foreign_key'
+    field_template = '%(column_type)s%(nullable)s REFERENCES "%(to_table)s" ("%(to_pk)s")'
     
     def __init__(self, to, null=False, related_name=None, *args, **kwargs):
         self.to = to
-        self.null = null
         self.related_name = related_name
-        kwargs['to_table'] = to._meta.db_table
-        if null:
-            kwargs['nullable'] = ''
-        else:
-            kwargs['nullable'] = 'NOT NULL'
-        super(ForeignKeyField, self).__init__(*args, **kwargs)
+        kwargs.update({
+            'to_table': to._meta.db_table,
+            'to_pk': to._meta.pk_name
+        })
+        super(ForeignKeyField, self).__init__(null=null, *args, **kwargs)
     
     def add_to_class(self, klass, name):
         self.descriptor = name
         self.name = name + '_id'
+        self.model = klass
+        
         if self.related_name is None:
             self.related_name = klass._meta.db_table + '_set'
         
@@ -862,73 +1025,86 @@ class ForeignKeyField(IntegerField):
     
     def lookup_value(self, lookup_type, value):
         if isinstance(value, Model):
-            return value.id
+            return value.get_pk()
         return value or None
 
 
+class BaseModelOptions(object):
+    def __init__(self, model_class, options=None):
+        self.rel_fields = {}
+        self.fields = {}
+        self.model_class = model_class
+        
+        # configurable options
+        options = options or {}
+        self.database = options.get('database', database)
+    
+    def get_field_by_name(self, name):
+        if name in self.fields:
+            return self.fields[name]
+        raise AttributeError('Field named %s not found' % name)
+    
+    def get_related_field_by_name(self, name):
+        if name in self.rel_fields:
+            return self.fields[self.rel_fields[name]]
+    
+    def get_related_field_for_model(self, model, name=None):
+        for field in self.fields.values():
+            if isinstance(field, ForeignKeyField) and field.to == model:
+                if name is None or name == field.name or name == field.descriptor:
+                    return field
+    
+    def get_reverse_related_field_for_model(self, model, name=None):
+        for field in model._meta.fields.values():
+            if isinstance(field, ForeignKeyField) and field.to == self.model_class:
+                if name is None or name == field.name or name == field.descriptor:
+                    return field
+    
+    def rel_exists(self, model):
+        return self.get_related_field_for_model(model) or \
+               self.get_reverse_related_field_for_model(model)
+
+
 class BaseModel(type):
     def __new__(cls, name, bases, attrs):
         cls = super(BaseModel, cls).__new__(cls, name, bases, attrs)
 
-        meta_attrs = attrs.pop('Meta', None)
-
-        class Meta(object):
-            rel_fields = {}
-            fields = {}
-            database = database
-            
-            def __init__(self, model_class):
-                self.model_class = model_class
-            
-            def get_field_by_name(self, name):
-                if name in self.fields:
-                    return self.fields[name]
-                raise AttributeError('Field named %s not found' % name)
-            
-            def get_related_field_by_name(self, name):
-                if name in self.rel_fields:
-                    return self.fields[self.rel_fields[name]]
-            
-            def get_related_field_for_model(self, model, name=None):
-                for field in self.fields.values():
-                    if isinstance(field, ForeignKeyField) and field.to == model:
-                        if name is None or name == field.name:
-                            return field
-            
-            def get_reverse_related_field_for_model(self, model, name=None):
-                for field in model._meta.fields.values():
-                    if isinstance(field, ForeignKeyField) and field.to == self.model_class:
-                        if name is None or name == field.name:
-                            return field
-            
-            def rel_exists(self, model):
-                return self.get_related_field_for_model(model) or \
-                       self.get_reverse_related_field_for_model(model)
-            
+        attr_dict = {}
+        meta = attrs.pop('Meta', None)
+        if meta:
+            attr_dict = meta.__dict__
         
-        _meta = Meta(cls)
-
-        if meta_attrs:
-            for k, v in meta_attrs.__dict__.items():
-                setattr(_meta, k, v)
+        for b in bases:
+            base_meta = getattr(b, '_meta', None)
+            if not base_meta:
+                continue
+            
+            for (k, v) in base_meta.__dict__.items():
+                if not k.startswith('_') and k not in attr_dict:
+                    attr_dict[k] = v
+        
+        _meta = BaseModelOptions(cls, attr_dict)
+        
+        if not hasattr(_meta, 'db_table'):
+            _meta.db_table = re.sub('[^a-z]+', '_', cls.__name__.lower())
 
         setattr(cls, '_meta', _meta)
         
-        _meta.db_table = re.sub('[^a-z]+', '_', cls.__name__.lower())
-        
-        has_primary_key = False
+        _meta.pk_name = None
 
         for name, attr in cls.__dict__.items():
             if isinstance(attr, Field):
                 attr.add_to_class(cls, name)
                 _meta.fields[attr.name] = attr
                 if isinstance(attr, PrimaryKeyField):
-                    has_primary_key = True
+                    _meta.pk_name = attr.name
         
-        if not has_primary_key:
+        if _meta.pk_name is None:
+            _meta.pk_name = 'id'
             pk = PrimaryKeyField()
-            pk.add_to_class(cls, 'id')
-            _meta.fields['id'] = pk
+            pk.add_to_class(cls, _meta.pk_name)
+            _meta.fields[_meta.pk_name] = pk
+            
                 
         if hasattr(cls, '__unicode__'):
             setattr(cls, '__repr__', lambda self: '<%s: %s>' % (
@@ -945,7 +1121,9 @@ class Model(object):
             setattr(self, k, v)
     
     def __eq__(self, other):
-        return other.__class__ == self.__class__ and self.id and other.id == self.id
+        return other.__class__ == self.__class__ and \
+               self.get_pk() and \
+               other.get_pk() == self.get_pk()
     
     def get_field_dict(self):
         field_val = lambda f: (f.name, getattr(self, f.name))
@@ -955,6 +1133,14 @@ class Model(object):
     @classmethod
     def create_table(cls):
         cls._meta.database.create_table(cls)
+        
+        for field_name, field_obj in cls._meta.fields.items():
+            if isinstance(field_obj, PrimaryKeyField):
+                cls._meta.database.create_index(cls, field_obj.name, True)
+            elif isinstance(field_obj, ForeignKeyField):
+                cls._meta.database.create_index(cls, field_obj.name)
+            elif field_obj.db_index:
+                cls._meta.database.create_index(cls, field_obj.name)
     
     @classmethod
     def drop_table(cls):
@@ -994,12 +1180,18 @@ class Model(object):
     def get(cls, *args, **kwargs):
         return cls.select().where(*args, **kwargs).paginate(1, 1).execute().next()
     
+    def get_pk(self):
+        return getattr(self, self._meta.pk_name, None)
+    
     def save(self):
         field_dict = self.get_field_dict()
-        field_dict.pop('id')
-        if self.id:
-            update = self.update(**field_dict).where(id=self.id)
+        field_dict.pop(self._meta.pk_name)
+        if self.get_pk():
+            update = self.update(
+                **field_dict
+            ).where(**{self._meta.pk_name: self.get_pk()})
             update.execute()
         else:
             insert = self.insert(**field_dict)
-            self.id = insert.execute()
+            new_pk = insert.execute()
+            setattr(self, self._meta.pk_name, new_pk)
